"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const koa_1 = __importDefault(require("koa"));
const koa_body_1 = __importDefault(require("koa-body"));
const weda_node_sdk_1 = require("@cloudbase/weda-node-sdk");
const routes_1 = __importStar(require("./routes"));
const config_1 = require("./config");
const call_datasource_1 = require("./fns/call-datasource");
const transform_ds_1 = require("./fns/load-datasources/transform-ds");
const app = new koa_1.default();
/**
 * 统一通用处理
 *  1. 成功的返回结果添加 code/data 包装
 *  2. 失败后, 失败的包装
 */
const generalHandler = async function (ctx, next) {
    try {
        await next();
        // POST 为正常请求接口的方法
        if (ctx.method === 'POST' && ctx.path !== `/${routes_1.ROUTE_NAMES.graphql}`) {
            ctx.body = {
                code: 0,
                data: ctx.body,
            };
        }
    }
    catch (err) {
        if (config_1.IS_DEV)
            console.warn(err);
        // TODO: status恢复为200, body.code 错误代码标准化
        ctx.status = err.statusCode || err.status || 500;
        ctx.body = {
            code: 1,
            message: err.message,
        };
    }
};
/**
 * 更新数据源 sdk 所需的公共context信息
 */
const updateCommonContext = function (ctx, next) {
    const requestBody = ctx.request.body || {};
    const { testDataSource } = requestBody;
    weda_node_sdk_1.setCommonContext({
        callDataSource: call_datasource_1.callDataSource,
        // @ts-ignore
        rawContext: ctx.req._scfContext || (config_1.IS_DEV && global.TEST_CONTEXT),
        entrance: getCallEntrance(ctx),
        cfnParams: Object.assign(Object.assign(Object.assign({}, ctx.query), requestBody), { 
            // @ts-ignore
            testDataSource: testDataSource && transform_ds_1.standardizeDataSource(testDataSource) }),
    });
    return next();
};
/**
 * 获取入口处调用的数据源名称及方法名
 */
function getCallEntrance(ctx) {
    const paths = ctx.path.replace('/', '').split('/');
    const category = paths[0];
    if (category === routes_1.ROUTE_NAMES.common)
        return;
    if (category === routes_1.ROUTE_NAMES.datasource) {
        return { dataSourceName: paths[1], methodName: paths[2] };
    }
    // TODO 未处理 graphql 的情况
}
app.use(generalHandler);
app.use(koa_body_1.default());
app.use(updateCommonContext);
routes_1.default.forEach(route => app.use(route));
if (config_1.IS_DEV) {
    const port = process.env.PORT || 8080;
    app.listen(port, () => {
        console.log(`Local server start at http://localhost:${port}/`);
    });
}
exports.default = app;
