"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTypeFromSchema = void 0;
const graphql_1 = require("graphql");
const scalar_1 = require("./scalar");
// import { callDataSource } from '../../call-datasource';
const SCALAR_TYPES = {
    number: graphql_1.GraphQLFloat,
    boolean: graphql_1.GraphQLBoolean,
    string: graphql_1.GraphQLString,
};
// let idx = 0;
/**
 * 根据数据源的schema 获取graphql 的类型定义
 * @param schema 数据源字段 schema
 * @param needRelated 是否需要关联的字段信息(比如枚举, 关联引用等)
 */
function getTypeFromSchema(typeName, schema, needRelated) {
    // @ts-ignore
    const scalarType = SCALAR_TYPES[schema.type];
    if (scalarType)
        return scalarType;
    if (schema.type === 'array') {
        return new graphql_1.GraphQLList(getTypeFromSchema(typeName + (schema['x-id'] || ''), schema.items, needRelated));
    }
    if (schema.type === 'object') {
        if (!Object.keys(schema.properties).length)
            return scalar_1.GraphQLAny;
        return new graphql_1.GraphQLObjectType({
            // eslint-disable-next-line no-plusplus
            name: typeName,
            description: schema.title,
            fields: () => Object.keys(schema.properties).reduce((acc, cur) => {
                const keyProp = schema.properties[cur];
                acc[cur] = {
                    description: keyProp.title,
                    type: getTypeFromSchema(typeName + (keyProp['x-id'] || ''), keyProp, needRelated),
                };
                if (needRelated) {
                    // 枚举字段
                    if (keyProp['x-enumObj']) {
                        acc[`${cur}__label`] = {
                            type: graphql_1.GraphQLString,
                            description: `${keyProp.title}枚举显示值`,
                            resolve: (parent) => {
                                const item = keyProp['x-enumObj'].find(it => it.value === parent[cur]);
                                return (item === null || item === void 0 ? void 0 : item.label) || '<unspecified>';
                            },
                        };
                    }
                }
                return acc;
            }, {}),
        });
    }
    throw new Error(`unsupported type ${schema.type} in datasource schema`);
}
exports.getTypeFromSchema = getTypeFromSchema;
