"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllBuiltinMethodName = exports.isGraphqlActionType = exports.getMethodOutTypeName = exports.getMethodInTypeName = exports.getRootFieldName = void 0;
const weda_node_sdk_1 = require("@cloudbase/weda-node-sdk");
/** 获取graphql方法名称  */
function getRootFieldName(dataSourceName, methodName) {
    return `${dataSourceName}${weda_node_sdk_1.capitalize(methodName)}`;
}
exports.getRootFieldName = getRootFieldName;
/** 获取graphql方法入参的类型名称 */
function getMethodInTypeName(dataSourceName, methodName, isBuiltin) {
    const rinsedMethodName = isBuiltin ? weda_node_sdk_1.normalizeBuiltinMethodName(methodName) : methodName;
    return `${getRootFieldName(dataSourceName, rinsedMethodName)}Payload`;
}
exports.getMethodInTypeName = getMethodInTypeName;
/** 获取graphql方法出参的类型名称 */
function getMethodOutTypeName(dataSourceName, methodName, isBuiltin) {
    const rinsedMethodName = isBuiltin ? weda_node_sdk_1.normalizeBuiltinMethodName(methodName) : methodName;
    return `${getRootFieldName(dataSourceName, rinsedMethodName)}Response`;
}
exports.getMethodOutTypeName = getMethodOutTypeName;
/**
 * 当前方法是否为 graphql 指定的方法类型
 *  @param method 字符串则是内置方法名, 对象则为自定义方法
 */
// eslint-disable-next-line @typescript-eslint/no-unused-vars
function isGraphqlActionType(type, method) {
    return true;
}
exports.isGraphqlActionType = isGraphqlActionType;
const OLD_BUILTIN_METHOD_NAMES = ['getList', 'create', 'delete', 'getItem', 'update', 'getRecords'];
/**
 * 获取带weda前缀及不带前缀的内置方法名称
 * @param names 内置方法名
 */
function getAllBuiltinMethodName(names) {
    if (!names || !names.length)
        return [];
    // 转换为不带 weda 前缀的名称并且名称包含在旧的内置方法名称中
    //  这些名称需要做名称 weda 是否加前缀的处理, 其他新增方法无此问题, 不必兼容
    const oldNames = names
        .map(name => name.replace(/^weda([A-Z])/, ($0, $1) => $1.toLowerCase()))
        .filter(n => OLD_BUILTIN_METHOD_NAMES.includes(n));
    // 旧的数据源描或旧的应用模版带的数据源述中可能没有 getRecords 方法, 在此 保证内置数据源一定可以使用 getRecords
    if (oldNames.indexOf('getRecords') === -1)
        oldNames.push('getRecords');
    // 带 weda 前缀的名称
    const newNames = oldNames.map(name => `weda${name.charAt(0).toUpperCase()}${name.slice(1)}`);
    const allNames = names.concat(oldNames.concat(newNames));
    // 名称去重
    return allNames.filter((n, idx) => allNames.indexOf(n) === idx);
}
exports.getAllBuiltinMethodName = getAllBuiltinMethodName;
